<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| Copyright (c) 2000-2003 the phpAdsNew developers                          |
| For contact details, see: http://www.phpadsnew.com/                       |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: publisherhourofday.plugin.php 1423 2005-02-02 13:18:58Z michal $
*/

include_once('lib.php');

// Public name of the plugin info function
$plugin_info_function		= "Plugin_PublisherHourOfDayAnalysisInfo";


// Public info function
function Plugin_PublisherHourOfDayAnalysisInfo()
{	
	$plugininfo = array (
		"plugin-name"			=> $GLOBALS['strPublisherHourOfDayAnalysisReport'],
		"plugin-description"	=> $GLOBALS['strPublisherHourOfDayAnalysisDescription'],
		'plugin-category'     => 'publisher',
		"plugin-author"		=> "Michal Pawlowski",
		"plugin-export"		=> "csv",
		"plugin-authorize"	=> phpAds_Admin+phpAds_Agency+phpAds_Affiliate,
		"plugin-execute"	=> "Plugin_PublisherHourOfDayAnalysisExecute",
		"plugin-import"		=> array (
			"campaignid"	=> array (
				"title"		=> $GLOBALS['strAffiliate'],
				"type"		=> "affiliateid-dropdown" ),
			"startdate"		=> array (
                "title"		=> "Start Date",
                "type"		=> "edit",
                "size"		=> 10,
                "default"	=> date("Y/m/d",mktime (0,0,0,date("m")-1,1,  date("Y")))
            ),
            "enddate"		=> array (
                "title"		=> "End Date",
                "type"		=> "edit",
                "size"		=> 10,
                "default"	=> date("Y/m/d",mktime (0,0,0,date("m"),1,  date("Y")))
            ),
		 )
	);
	
	return ($plugininfo);
}

/*-------------------------------------------------------*/
/* Private plugin function                               */
/*-------------------------------------------------------*/

function Plugin_PublisherHourOfDayAnalysisExecute($affiliateid, $startdate, $enddate)
{
    
    require_once 'Spreadsheet/Excel/Writer.php';
        
	global $phpAds_config, $date_format;
	global $strAffiliate;
	
	$reportName = $GLOBALS['strPublisherHourOfDayAnalysisReport'];
	
	$startDate = date(str_replace('%','',$date_format), strtotime($startdate));
	$endDate   = date(str_replace('%','',$date_format), strtotime($enddate));
	$dbStart = date('Y-m-d', strtotime($startdate));
	$dbEnd = date('Y-m-d', strtotime($enddate));
	// creating report object
    $workbook = new Spreadsheet_Excel_Writer();
    
    $worksheet = generateXLSHeader($workbook, $reportName, $startDate, $endDate,
                                   $affiliateid, $GLOBALS['strPublisherHourOfDayAnalysisDescription']);
    // where to start printing structural data
    $rowStart = 7;
    $columnStart = 0;
    
    // formatting - begin
    $formatInteger =& $workbook->addFormat();
    $formatInteger->setNumFormat($GLOBALS['excell_integer_formatting']);
    
    $formatDecimal =& $workbook->addFormat();
    $formatDecimal->setNumFormat($GLOBALS['excell_decimal_formatting']);
    
    $columnHeader =& $workbook->addFormat();
    $columnHeader->setAlign('center');
    $columnHeader->setBold();
    // formatting - end
    
    $formatRight =& $workbook->addFormat();
    $formatRight->setAlign('right');
    
    $query = "SELECT hour
                  ,c.priority
                  ,coalesce(sum(ds.requests),0)        as requests
                  ,coalesce(sum(ds.impressions),0)     as impressions
                  ,coalesce(sum(ds.clicks),0)          as clicks
                  ,coalesce(sum(ds.conversions),0)     as conversions
              FROM zones z
                  ,data_summary_ad_hourly ds
                  ,banners b
                  ,campaigns c
             WHERE z.zoneid = ds.zone_id
               AND ds.ad_id = b.bannerid
               AND b.campaignid = c.campaignid
               AND z.affiliateid = ".$affiliateid."
               AND ds.day >= '$dbStart'
               AND ds.day <= '$dbEnd'
             GROUP BY hour, c.priority
             ORDER BY c.priority ASC";
    
	$res = phpAds_dbQuery($query) or phpAds_sqlDie();
	
	if (phpAds_dbNumRows($res)) {
    	while ($row = phpAds_dbFetchArray($res)) {
    	    $data[] = $row;
    	}
    	
        $numrows = (is_array($data)) ? count($data): 0;
        
        // preparing result data
    	$dataSorted = Plugin_PublisherHourOfDayAnalysisPrepareData($data);
    	
    	$priorityNames = array('l' => $GLOBALS['strPriorityLow'],
    	                       'm' => $GLOBALS['strPriorityMedium'],
    	                       'h' => $GLOBALS['strPriorityHigh'],
    	                       't' => "All campaigns");
    	$previousPriority = '';
    	$previousStart = $rowStart;
    	
    	$i = 0; $currentRow = $rowStart;
    	foreach ($dataSorted as $block) {
    	   
    	   // wind what is the priority of this block
    	   if (list(,$row) = each($block)) {
    	       $worksheet->write($currentRow,$columnStart,
    	                         $priorityNames[$row['priority']], $columnHeader);
    	   }
    	   $currentRow++; // next row
    	   
           $worksheet->write($currentRow,$columnStart    ,
                             $GLOBALS['strHour']       , $columnHeader);
           $worksheet->write($currentRow,$columnStart + 1,
                             $GLOBALS['strRequests']   , $columnHeader);
           $worksheet->write($currentRow,$columnStart + 2,
                             $GLOBALS['strViews']      , $columnHeader);
           $worksheet->write($currentRow,$columnStart + 3,
                             $GLOBALS['strViewsRequestsRatio'], $columnHeader);
           $worksheet->write($currentRow,$columnStart + 4,
                             $GLOBALS['strClicks']     , $columnHeader);
           $worksheet->write($currentRow,$columnStart + 5,
                             $GLOBALS['strCTRShort']   , $columnHeader);
           $worksheet->write($currentRow,$columnStart + 6,
                             $GLOBALS['strConversions'], $columnHeader);
           $worksheet->write($currentRow,$columnStart + 7,
                             $GLOBALS['strCNVRShort']  , $columnHeader);
           
           $currentRow++;
    	   foreach ($block as $rowId => $row) {
    	       if (is_numeric($row['hour'])) {
    	           $row['hour'] = $row['hour'] . "::00";
    	       }
    	       $worksheet->write($currentRow, $columnStart, $row['hour']);
        	   $worksheet->writeNumber($currentRow, $columnStart + 1,
        	                           $row['requests'], $formatInteger);
    	       $worksheet->writeNumber($currentRow, $columnStart + 2,
    	                               $row['impressions'], $formatInteger);
    	       if ($row['requests'] != 0) {
    	           $worksheet->writeNumber($currentRow, $columnStart + 3,
    	                                   ($row['impressions'] / $row['requests']) * 100, $formatDecimal);
    	       } else {
    	           $worksheet->write($currentRow, $columnStart + 3,"N/A");
    	       }
    	       $worksheet->writeNumber($currentRow, $columnStart + 4,
    	                               $row['clicks'], $formatInteger);
    	       if ($row['impressions'] != 0) {
    	           $worksheet->writeNumber($currentRow, $columnStart + 5,
    	                                   ($row['clicks'] / $row['impressions']) * 100, $formatDecimal);
    	       } else {
    	           $worksheet->write($currentRow, $columnStart + 5,"N/A");
    	       }
    	       $worksheet->writeNumber($currentRow, $columnStart + 6,
    	                               $row['conversions'],$formatInteger);
    	       if ($row['impressions'] != 0) {
    	           $worksheet->writeNumber($currentRow, $columnStart + 7,
    	                                   ($row['conversions'] / $row['clicks']) * 100, $formatDecimal);
    	       } else {
    	           $worksheet->write($currentRow, $columnStart + 7,"N/A");
    	       }
    	    
    	       $currentRow++;
    	   }
    	   
    	   $currentRow++;
    	   $i++;
    	}
	} else {
	    $worksheet->write($rowStart, $columnStart,
	                      $GLOBALS['strNoData'], $columnHeader);
	}

    $workbook->close();
}

/*----------------------------------------*/
/* Function for sorting result data       */
/* and splitting into blocks result array */
/*----------------------------------------*/
function Plugin_PublisherHourOfDayAnalysisPrepareData(& $data)
{
    $retArray = array();
    
    // creating block for totals for each day of the week
    $totalBlock = array();
    
    $blockNr = 1;
    $previousPriority = '';
    
    // Sorting Array by priority
    foreach ($data as $row) {
        if ($previousPriority != $row['priority']) {
            $previousPriority = $row['priority'];
            $blockIndex = 0;
            $blockNr++;
        }
        
        $retArray[$blockNr][$row['hour']] = $row;
        $totalBlock[$row['hour']]['hour']      = $row['hour'];
        $totalBlock[$row['hour']]['priority']     = "t";
        $totalBlock[$row['hour']]['requests']    += $row['requests'];
        $totalBlock[$row['hour']]['impressions'] += $row['impressions'];
        $totalBlock[$row['hour']]['clicks']      += $row['clicks'];
        $totalBlock[$row['hour']]['conversions'] += $row['conversions'];
        $blockIndex++;
    }
    
    $retArray[0] = $totalBlock;
    
    // Sorting array by day of the week
    ksort($retArray);
    
    foreach ($retArray as $blockId => $block) {
        ksort($retArray[$blockId]);
        addTotals($retArray[$blockId]);
    }
    
    return $retArray;
}

?>
