<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: lib-upgrade.inc.php 2031 2005-03-11 09:28:52Z andrew $
*/

/**
 * @package    Max
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once MAX_LIB . '/common.php';
require_once phpAds_path . '/admin/lib-zones.inc.php'; // Needed for the constants at the top of the file...
require_once phpAds_path . '/libraries/dba.php';

/**
 * A class to perform upgrades from phpAdsNew 2.0 or greater, and from previous
 * version of Max.
 */
class Upgrade
{
    var $prefix;
    var $dbConnection;
    var $tables;
    var $tableType;
    var $upgradeFrom;
    var $upgradeTo;
    
    /**
     * The class constructor method.
     *
     * @param string $prefix The table prefix to use for database tables.
     * @param mixed $dbConnection A reference to an existing MySQL connection
     *                            to the admin database.
     */
    function Upgrade($prefix, $dbConnection)
    {
        // Set time limit and ignore user abort, as upgrade can take some itme
        if (!ini_get('safe_mode')) {
            @set_time_limit(600);
            @ignore_user_abort(true);
        }
        $this->prefix = $prefix;
        $this->dbConnection = $dbConnection;
        $this->tables = phpAds_prepareDatabaseStructure();
        global $phpAds_config;
        $this->tableType = $phpAds_config['table_type'];
    }
    
    /**
     * A method to determine if an older version of Max (or phpAdsNew)
     * is currently installed.
     *
     * @param string $version The version of Max currently being installed.
     * @return boolean True if a previous version of Max is installed, 
     *                 false otherwise.
     */
    function previousVersionExists($version)
    {
        $this->upgradeTo = $version;
        // Does the application variables table exists?
        $query = 'DESCRIBE ' . $this->prefix . 'application_variable';
        $result = mysql_query($query, $this->dbConnection);
        if (!$result) {
            // Could not find the application variables table,
            // so definately need to upgrade
            return true;
        } else {
            // What is the current version of Max that is installed?
            $query = '
                SELECT
                    value AS max_version
                FROM
                    ' . $this->prefix . 'application_variable
                WHERE
                    name = \'max_version\'';
            $result = mysql_query($query, $this->dbConnection);
            if ($result) {
                $row = mysql_fetch_array($result);
                $this->upgradeFrom = $row['max_version'];
                return $this->_compareVersions($this->upgradeTo, $this->upgradeFrom);
            }
        }
        // Versions were the same, or less, or there was an error.
        // Don't upgrade.
        return false;
    }
    
    /**
     * A method to perform the necessary upgrade steps to update Max
     * to the necessary database format.
     *
     * @param array $errors A reference to an array to have errors added to it.
     */
    function upgradeDatabase(&$errors)
    {
        $errors = array();
        // Is the upgrageFrom variable defined?
        if (isset($this->upgradeFrom)) {
            // We are upgrading from Max after v0.1.16-beta,
            // so just do the required upgrade actions
            if ($this->_compareVersions('v0.2.0-alpha', $this->upgradeFrom)) {
                // Upgrade to v0.2.0-alpha
                $this->_upgradeToTwoZeroAlpha($errors);
            }
            if ($this->_compareVersions('v0.2.1-alpha', $this->upgradeFrom)) {
                // Upgrade to v0.2.1-alpha
                $this->_upgradeToTwoOneAlpha($errors);
            }
            if ($this->_compareVersions('v0.2.3-alpha', $this->upgradeFrom)) {
                // Upgrade to v0.2.3-alpha
                $this->_upgradeToTwoThreeAlpha($errors);
            }
            if ($this->_compareVersions('v0.2.4-alpha', $this->upgradeFrom)) {
                // Upgrade to v0.2.4-alpha
                $this->_upgradeToTwoFourAlpha($errors);
            }
        } else {
            // Perfom *all* possible upgrade actions, in order
            $this->_upgradeEarly($errors);           // Upgrade to v0.1.16-beta
            $this->_upgradeToTwoZeroAlpha($errors);  // Upgrade to v0.2.0-alpha
            $this->_upgradeToTwoOneAlpha($errors);   // Upgrade to v0.2.1-alpha
            $this->_upgradeToTwoThreeAlpha($errors); // Upgrade to v0.2.3-alpha
            $this->_upgradeToTwoFourAlpha($errors);  // Upgrade to v0.2.4-alpha
        }
        if (count($errors) == 0) {
            // Always upgrade the installed version number
            $this->_upgradeInstalledVersion();
        }
    }
    
    /**
     * A private method for comparing version numbers.
     *
     * @private
     * @param string $first The first version number.
     * @param string $second The second version number.
     * @return boolean True if the first version number is greater than the second,
     *                 false otherwise.
     */
    function _compareVersions($first, $second)
    {
        if ((!isset($first)) || (!isset($second))) {
            return false;
        }
        // Obtain the parts of the verison numbers
        if (preg_match('/(\d+)\.(\d+)\.(\d+)(?:-([a-z]+))?/', $first, $matches)) {
            $firstMajor = $matches[1];
            $firstMinor = $matches[2];
            $firstPatch = $matches[3];
            $firstType  = $matches[4];
        }
        if (preg_match('/(\d+)\.(\d+)\.(\d+)(?:-([a-z]+))?/', $second, $matches)) {
            $secondMajor = $matches[1];
            $secondMinor = $matches[2];
            $secondPatch = $matches[3];
            $secondType  = $matches[4];
        }
        // Compare the major versions
        if (isset($firstMajor) && isset($secondMajor) && ($firstMajor > $secondMajor)) {
            return true;
        }
        // Compare the minor versions
        if (isset($firstMajor) && isset($secondMajor) && ($firstMajor == $secondMajor)) {
            if (isset($firstMinor) && isset($secondMinor) && ($firstMinor > $secondMinor)) {
                return true;
            }
        }
        // Compare the patch levels
        if (isset($firstMajor) && isset($secondMajor) && ($firstMajor == $secondMajor)) {
            if (isset($firstMinor) && isset($secondMinor) && ($firstMinor == $secondMinor)) {
                if (isset($firstPatch) && isset($secondPatch) && ($firstPatch > $secondPatch)) {
                    return true;
                }
            }
        }
        // Compare the release types
        if (isset($firstMajor) && isset($secondMajor) && ($firstMajor == $secondMajor)) {
            if (isset($firstMinor) && isset($secondMinor) && ($firstMinor == $secondMinor)) {
                if (isset($firstPatch) && isset($secondPatch) && ($firstPatch == $secondPatch)) {
                    if (isset($firstType) && isset($secondType)) {
                        if (($secondType == 'alpha') && ($firstType != 'alpha')) {
                            return true;
                        } else if (($secondType == 'beta') && (($firstType != 'alpha') || ($firstType != 'beta'))) {
                            return true;
                        } else if (($secondType == 'rc') && ($firstType == 'stable')) {
                            return true;
                        }
                    }
                }
            }
        }
        // Version was the same, less
        return false;
    }
    
    /**
     * A private method for updating the version number
     *
     * @private
     * @param array $errors A reference to an array to have errors added to it.
     */
    function _upgradeInstalledVersion(&$errors)
    {
        $query = 'UPDATE ' . $this->prefix . 'application_variable SET value = \'' . $this->upgradeTo . ' \' WHERE name = \'max_version\'';
        $result = mysql_query($query, $this->dbConnection);
        if (!$result) {
            $errors[] = "Error executing upgrade code: $query";
        }
    }
    
    /**
     * A private method to upgrade the database from the v0.2.3-alpha
     * format to the v0.2.4-alpha format.
     *
     * @private
     */
    function _upgradeToTwoFourAlpha(&$errors)
    {
        $queries = array();
        $queries[] = "ALTER TABLE {$this->prefix}config DROP COLUMN table_border_color";
        $queries[] = "ALTER TABLE {$this->prefix}config DROP COLUMN table_back_color";
        $queries[] = "ALTER TABLE {$this->prefix}config DROP COLUMN table_back_color_alternative";
        $queries[] = "ALTER TABLE {$this->prefix}config DROP COLUMN main_back_color";
        $queries[] = "ALTER TABLE {$this->prefix}config ADD COLUMN gui_header_background_color VARCHAR(7) DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->prefix}config ADD COLUMN gui_header_foreground_color VARCHAR(7) DEFAULT ''";
        $this->_runQueries($queries, $errors);
    }
    
    /**
     * A private method to upgrade the database from the v0.2.1-alpha
     * format to the v0.2.3-alpha format.
     *
     * @private
     */
    function _upgradeToTwoThreeAlpha(&$errors)
    {
        // Fix the config table's gui_link_compact_limit field
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE gui_link_compact_limit gui_link_compact_limit INTEGER DEFAULT \'50\'';
        $this->_runQueries($queries, $errors);    

        // Create the ad_zone_assoc table
        $key = $this->prefix . 'ad_zone_assoc';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the placement_zone_assoc table
        $key = $this->prefix . 'placement_zone_assoc';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the category table
        $key = $this->prefix . 'category';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the ad_category table
        $key = $this->prefix . 'ad_category_assoc';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Transfer keywords from the banners table to the category table
        $query = "SELECT bannerid, keyword FROM {$this->prefix}banners WHERE keyword != ''";
        $result = mysql_query($query, $this->dbConnection) or die($query);
        while ($row = mysql_fetch_array($result)) {
        	$adId = $row['bannerid'];
        	$keywords = split(' ', $row['keyword']);
        	foreach($keywords as $keyword) {
	        	$query2 = "SELECT category_id FROM {$this->prefix}category WHERE name='$keyword'";
	        	$result2 = mysql_query($query2, $this->dbConnection) or die($query2);
	        	if ($row2 = mysql_fetch_array($result2)) {
	        		$categoryId = $row2['category_id'];
	        	} else {
		        	$categoryId = MAX_addCategory(array('name' => $keyword));
	        	}
	        	MAX_addAdCategory(array('ad_id' => $adId, 'category_id' => $categoryId));
        	}
        }
        
        // Transfer the contents of the 'what' column to the ad_zone_assoc and placement_zone_assoc tables.
        $query = "SELECT zoneid,what FROM {$this->prefix}zones WHERE what != ''";
        $result = mysql_query($query, $this->dbConnection) or die($query);
        while ($row = mysql_fetch_array($result)) {
        	$zoneId = $row['zoneid'];
        	$keywords = split(',', $row['what']);
        	$newkeywords = array();
        	foreach($keywords as $keyword) {
        		if (substr($keyword, 0, 9) == 'bannerid:') {
        			$adId = substr($keyword, 9);
	                MAX_addAdZone(array('zone_id' => $zoneId, 'ad_id' => $adId));
        		}
        		elseif (substr($keyword, 0, 11) == 'campaignid:') {
        			$placementId = substr($keyword, 11);
	                MAX_addPlacementZone(array('zone_id' => $zoneId, 'placement_id' => $placementId));
		        	MAX_addLinkedAdsToZone($zoneId, $placementId);
        		}
        		else {
        			$newkeywords[] = $keyword;
        		}
        	}
        	$keywords = addslashes(implode(',',$newkeywords));
        	$query3 = "UPDATE {$this->prefix}zones SET what='$keywords' WHERE zoneid=$zoneId";
        	$result3 = mysql_query($query3, $this->dbConnection) or die($query3);
        }
        // Always have a '0' node in ad_zone for each banner so that direct selection works.
        $query = "INSERT INTO {$this->prefix}ad_zone_assoc (ad_id,zone_id) SELECT bannerid,0 FROM {$this->prefix}banners";
        $result = mysql_query($query, $this->dbConnection) or die($query);
        
        $queries = array();
        $queries[] = "ALTER TABLE {$this->prefix}config DROP COLUMN gui_link_compact_limit";
        $queries[] = "ALTER TABLE {$this->prefix}banners DROP COLUMN keyword";
        $queries[] = "ALTER TABLE {$this->prefix}banners DROP COLUMN priority";
        $queries[] = "ALTER TABLE {$this->prefix}zones CHANGE what ad_selection TEXT NOT NULL DEFAULT ''";
        $this->_runQueries($queries, $errors);
    }
    
    /**
     * A private method to upgrade the database from the v0.2.0-alpha
     * format to the v0.2.1-alpha format.
     *
     * @private
     */
    function _upgradeToTwoOneAlpha(&$errors)
    {
        // Add support for ad request summarising
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'data_intermediate_ad ADD COLUMN requests INTEGER UNSIGNED NOT NULL DEFAULT 0 AFTER zone_id';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'data_summary_ad_hourly ADD COLUMN requests INTEGER UNSIGNED NOT NULL DEFAULT 0 AFTER zone_id';
        $this->_runQueries($queries, $errors);
    }
    
    /**
     * A private method to upgrade the database from the v0.1.x format
     * to the v0.2.0-alpha format.
     *
     * @private
     */
    function _upgradeToTwoZeroAlpha(&$errors)
    {
        // Add support for default anonymous campaigns to the config table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config ADD COLUMN gui_campaign_anonymous ENUM(\'t\',\'f\') AFTER gui_show_campaign_preview';
        $this->_runQueries($queries, $errors);
        
        // Create the data_intermediate_ad table
        $key = $this->prefix . 'data_intermediate_ad';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the data_intermediate_ad_connection table
        $key = $this->prefix . 'data_intermediate_ad_connection';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the data_intermediate_ad_variable_value table
        $key = $this->prefix . 'data_intermediate_ad_variable_value';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the data_raw_ad_click table
        $key = $this->prefix . 'data_raw_ad_click';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the data_raw_ad_impression table
        $key = $this->prefix . 'data_raw_ad_impression';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the data_raw_ad_request table
        $key = $this->prefix . 'data_raw_ad_request';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the data_raw_tracker_click table
        $key = $this->prefix . 'data_raw_tracker_click';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the data_raw_tracker_impression table
        $key = $this->prefix . 'data_raw_tracker_impression';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the data_raw_tracker_variable_value table
        $key = $this->prefix . 'data_raw_tracker_variable_value';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the data_summary_ad_hourly table
        $key = $this->prefix . 'data_summary_ad_hourly';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the data_summary_zone_impression_history table
        $key = $this->prefix . 'data_summary_zone_impression_history';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Create the log_maintenance_statistics table
        $key = $this->prefix . 'log_maintenance_statistics';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // Increase the size of details in the userlog table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'userlog CHANGE details details MEDIUMTEXT';
        $this->_runQueries($queries, $errors);
        
        // Drop the variabletype column from the variables table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'variables DROP variabletype';
        $this->_runQueries($queries, $errors);
        
        // Add the is_basket_value column to the variables table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'variables ADD is_basket_value INTEGER NOT NULL DEFAULT \'0\'';
        $this->_runQueries($queries, $errors);
    }
    
    /**
     * A private method to upgrade the database for all versions
     * prior to v0.1.16-beta. Note that in some cases, a check is
     * performed to test if the chages have already been made, as
     * the changes cannot be applied twice. In other cases, no check
     * is made, as the changes can simply be ignored by MySQL if
     * they are applied for a second time.
     *
     * @private
     * @param array $errors A reference to an array to have errors added to it.
     */
    function _upgradeEarly(&$errors)
    {
        // acls table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'acls CHANGE logical logical set(\'and\',\'or\') NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'acls CHANGE type type VARCHAR(16) NOT NULL DEFAULT \'\'';
        $this->_runQueries($queries, $errors);
        
        // adclicks table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adclicks DROP INDEX bannerid_date';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adclicks ADD userid VARCHAR(32) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adclicks CHANGE t_stamp t_stamp TIMESTAMP(14) NOT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adclicks CHANGE host host VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adclicks CHANGE source source VARCHAR(50) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adclicks CHANGE country country char(2) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adclicks ADD INDEX bannerid(bannerid)';
        $this->_runQueries($queries, $errors);
        
        // adconversions table
        $key = $this->prefix . 'adconversions';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // adstats table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adstats DROP PRIMARY KEY';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adstats DROP KEY bannerid_day';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adstats ADD conversions INT(11) NOT NULL DEFAULT \'0\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adstats CHANGE bannerid bannerid MEDIUMINT(9) NOT NULL DEFAULT \'0\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adstats CHANGE zoneid zoneid MEDIUMINT(9) NOT NULL DEFAULT \'0\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adstats ADD KEY day(day)';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adstats ADD KEY bannerid(bannerid)';
        $this->_runQueries($queries, $errors);
        
        // adviews table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adviews DROP INDEX bannerid_date';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adviews ADD userid VARCHAR(32) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adviews CHANGE t_stamp t_stamp TIMESTAMP(14) NOT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adviews CHANGE host host VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adviews CHANGE source source VARCHAR(50) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adviews CHANGE country country char(2) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'adviews ADD INDEX bannerid(bannerid)';
        $this->_runQueries($queries, $errors);

        // affiliates table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'affiliates ADD agencyid MEDIUMINT(9) NOT NULL DEFAULT \'0\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'affiliates ADD mnemonic VARCHAR(5) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'affiliates CHANGE name name VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'affiliates CHANGE contact contact VARCHAR(255) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'affiliates CHANGE email email VARCHAR(64) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'affiliates CHANGE website website VARCHAR(255) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'affiliates CHANGE username username VARCHAR(64) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'affiliates CHANGE password password VARCHAR(64) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'affiliates CHANGE permissions permissions MEDIUMINT(9) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'affiliates CHANGE language language VARCHAR(64) DEFAULT NULL';
        $this->_runQueries($queries, $errors);
        
        // agency table
        $key = $this->prefix . 'agency';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // application_variable table
        $key = $this->prefix . 'application_variable';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        $query = '
            INSERT INTO
                ' . $this->prefix . 'application_variable
                (name, value)
            VALUES
                (\'max_version\', \'' . $this->upgradeTo . '\')';
        $result = mysql_query($query, $this->dbConnection);
        if (!$result) {
            $errors[] = "Error executing upgrade code: $query";
        }
        
        // banners table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE clientid campaignid MEDIUMINT(9) NOT NULL DEFAULT \'0\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE filename filename VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE imageurl imageurl VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE htmltemplate htmltemplate TEXT NOT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE htmlcache htmlcache TEXT NOT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE target target VARCHAR(16) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE url url TEXT NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE alt alt VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE status status VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE keyword keyword VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE bannertext bannertext TEXT NOT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE description description VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners ADD adserver VARCHAR(50) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE compiledlimitation compiledlimitation TEXT NOT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners CHANGE append append TEXT NOT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners ADD alt_filename VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners ADD alt_imageurl VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners ADD alt_contenttype ENUM(\'gif\',\'jpeg\',\'png\') NOT NULL DEFAULT \'gif\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners ADD KEY campaignid (campaignid)';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners ADD capping INT(11) NOT NULL DEFAULT \'0\' AFTER block';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'banners ADD session_capping INT(11) NOT NULL DEFAULT \'0\' AFTER capping';
        $this->_runQueries($queries, $errors);
        
        // cache table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'cache CHANGE content content blob NOT NULL';
        $this->_runQueries($queries, $errors);
        
        // campaigns table
        $key = $this->prefix . 'campaigns';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        $query = '
            INSERT INTO
                ' . $this->prefix . 'campaigns
            SELECT
                clientid AS campaignid,
                clientname AS campaignname,
                parent AS clientid,
                views AS views,
                clicks AS clicks,
                \'-1\' AS conversions,
                expire AS expire,
                activate AS activate,
                active AS active,
                \'h\' AS priority,
                weight AS weight,
                target AS target,
                \'f\' AS optimise,
                \'f\' AS anonymous
            FROM
                ' . $this->prefix . 'clients
            WHERE
                parent > 0
        ';
        $result = mysql_query($query, $this->dbConnection);
        if (!$result) {
            // There was an error copying the data from the old phpAdsNew
            // clients table into the new Max campaigns table. If this was
            // because the parent column no longer exists in the client
            // table, this is okay, as we are obviously upgrading from
            // an early version of Max, and this has already been done.
            // If not, this is an error.
            if (mysql_errno($this->dbConnection) != 1054) {
                $errors[] = "Error executing upgrade code: $query";
            }
        } else {
            // The copy of data worked, so now update the data in the
            // campaigns table
            $query = 'UPDATE ' . $this->prefix . 'campaigns SET priority=\'l\' WHERE weight > 0 AND views = -1 AND clicks = -1 and conversions = -1';
            $result = mysql_query($query, $this->dbConnection);
            if (!$result) {
                $errors[] = "Error executing upgrade code: $query";
            }
        }
        
        // campaigns_trackers table
        $key = $this->prefix . 'campaigns_trackers';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // clients table
        $queries = array();
        $queries[] = 'DELETE FROM ' . $this->prefix . 'clients WHERE parent > 0';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients ADD agencyid MEDIUMINT(9) NOT NULL DEFAULT \'0\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients CHANGE clientname clientname VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients CHANGE contact contact VARCHAR(255) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients CHANGE email email VARCHAR(64) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients CHANGE clientusername clientusername VARCHAR(64) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients CHANGE clientpassword clientpassword VARCHAR(64) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients CHANGE permissions permissions MEDIUMINT(9) default NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients CHANGE language language VARCHAR(64) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients DROP active';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients DROP weight';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients DROP target';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'clients DROP parent';
        $this->_runQueries($queries, $errors);
        
        // config table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config DROP PRIMARY KEY';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE configid agencyid MEDIUMINT(9) DEFAULT \'0\' NOT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE my_header my_header VARCHAR(255) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE my_footer my_footer VARCHAR(255) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE language language VARCHAR(32) DEFAULT \'english\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE name name VARCHAR(32) default NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE override_gd_imageformat override_gd_imageformat VARCHAR(4) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE begin_of_week begin_of_week TINYINT(2) DEFAULT \'1\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE percentage_decimals percentage_decimals TINYINT(2) DEFAULT \'2\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE type_sql_allow type_sql_allow ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE type_url_allow type_url_allow ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE type_web_allow type_web_allow ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE type_html_allow type_html_allow ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE type_txt_allow type_txt_allow ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE type_web_mode type_web_mode TINYINT(2) DEFAULT \'0\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE type_web_dir type_web_dir VARCHAR(255) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE type_web_ftp type_web_ftp VARCHAR(255) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE type_web_url type_web_url VARCHAR(255) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE admin admin VARCHAR(64) DEFAULT \'phpadsuser\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE admin_pw admin_pw VARCHAR(64) default \'phpadspass\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE admin_fullname admin_fullname VARCHAR(255) DEFAULT \'Your Name\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE admin_email admin_email VARCHAR(64) DEFAULT \'your@email.com\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config ADD warn_admin ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config ADD warn_agency ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config ADD warn_client ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config ADD warn_limit MEDIUMINT(9) NOT NULL DEFAULT \'0\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE admin_email_headers admin_email_headers VARCHAR(64) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE admin_novice admin_novice ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE default_banner_weight default_banner_weight TINYINT(4) DEFAULT \'1\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE default_campaign_weight default_campaign_weight TINYINT(4) DEFAULT \'1\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE client_welcome client_welcome ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE client_welcome_msg client_welcome_msg TEXT';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE content_gzip_compression content_gzip_compression ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE userlog_email userlog_email ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE userlog_priority userlog_priority ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE userlog_autoclean userlog_autoclean ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE gui_show_campaign_info gui_show_campaign_info ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE gui_show_campaign_preview gui_show_campaign_preview ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE gui_show_banner_info gui_show_banner_info ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE gui_show_banner_preview gui_show_banner_preview ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE gui_show_banner_html gui_show_banner_html ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE gui_show_matching gui_show_matching ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE gui_show_parents gui_show_parents ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE gui_hide_inactive gui_hide_inactive ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE qmail_patch qmail_patch ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE updates_frequency updates_frequency TINYINT(2) DEFAULT \'7\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE updates_timestamp updates_timestamp INT(11) DEFAULT \'0\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE updates_last_seen updates_last_seen decimal(7,3) DEFAULT \'0.000\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE allow_invocation_plain allow_invocation_plain ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config ADD allow_invocation_plain_nocookies ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE allow_invocation_js allow_invocation_js ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE allow_invocation_frame allow_invocation_frame ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE allow_invocation_xmlrpc allow_invocation_xmlrpc ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE allow_invocation_local allow_invocation_local ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE allow_invocation_interstitial allow_invocation_interstitial ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE allow_invocation_popup allow_invocation_popup ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE auto_clean_tables auto_clean_tables ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE auto_clean_tables_interval auto_clean_tables_interval TINYINT(2) DEFAULT \'5\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE auto_clean_userlog auto_clean_userlog ENUM(\'t\',\'f\') DEFAULT \'f\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE auto_clean_userlog_interval auto_clean_userlog_interval TINYINT(2) DEFAULT \'5\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE auto_clean_tables_vacuum auto_clean_tables_vacuum ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE autotarget_factor autotarget_factor float DEFAULT \'-1\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config CHANGE maintenance_timestamp maintenance_timestamp INT(11) DEFAULT \'0\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config ADD compact_stats ENUM(\'t\',\'f\') DEFAULT \'t\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config ADD statslastday DATE NOT NULL DEFAULT \'0000-00-00\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config ADD statslasthour TINYINT(4) NOT NULL DEFAULT \'0\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'config ADD PRIMARY KEY(agencyid)';
        $this->_runQueries($queries, $errors);
        
        // conversionlog table
        $key = $this->prefix . 'conversionlog';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // images table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'images CHANGE filename filename VARCHAR(128) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'images CHANGE t_stamp t_stamp TIMESTAMP(14) NOT NULL';
        $this->_runQueries($queries, $errors);
        
        // log_maintenance table
        $key = $this->prefix . 'log_maintenance';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);

        // session table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'session CHANGE sessionid sessionid VARCHAR(32) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'session CHANGE lastused lastused TIMESTAMP(14) NOT NULL';
        $this->_runQueries($queries, $errors);

        // Sometimes phpAdsNew installs the targetstats table without
        // the correct prefix, and instead uses "tbl_" as the prefix.
        // Run some SQL to fix this, but ignore all errors.
        $key = $this->prefix . 'targetstats';
        $query = "ALTER TABLE tbl_targetstats RENAME $key";
        $result = mysql_query($query, $this->dbConnection);

        // targetstats table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'targetstats DROP PRIMARY KEY';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'targetstats CHANGE clientid campaignid MEDIUMINT(9) NOT NULL DEFAULT \'0\'';
        $this->_runQueries($queries, $errors);
        
        // trackers table
        $key = $this->prefix . 'trackers';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // userlog table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'userlog CHANGE details details TEXT';
        $this->_runQueries($queries, $errors);
        
        // variables table
        $key = $this->prefix . 'variables';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // variablevalues table
        $key = $this->prefix . 'variablevalues';
        phpAds_createTable($key, $this->tables[$key], $this->tableType);
        
        // zones table
        $queries = array();
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'zones CHANGE affiliateid affiliateid MEDIUMINT(9) DEFAULT NULL';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'zones CHANGE zonename zonename VARCHAR(245) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'zones CHANGE description description VARCHAR(255) NOT NULL DEFAULT \'\'';
        $queries[] = 'ALTER TABLE ' . $this->prefix . 'zones ADD COLUMN forceappend ENUM (\'t\', \'f\') DEFAULT \'f\'';
        $this->_runQueries($queries, $errors);
        // Fix the 'what' column which links campaigns (used to be clients)
        $query = 'SELECT * FROM ' . $this->prefix . 'zones';
        $result = mysql_query($query, $this->dbConnection);
        if (!$result) {
            $errors[] = "Error executing upgrade code: $query";
        } else {
            while ($row = mysql_fetch_array($result)) {
                $newWhat = preg_replace('/clientid:/', 'campaignid:', $row['what']);
                if ($newWhat!= $row['what']) {
                    $query = 'UPDATE ' . $this->prefix . 'zones SET what = \'' . $newWhat . '\' WHERE ';
                    foreach ($row as $key => $value) {
                        if (preg_match('/\d+/', $key)) {
                            continue;
                        } else if ($key == 'what') {
                            continue;
                        } else if ($value != '') {
                            $query .= "$key = '$value' AND ";
                        }
                    }
                    $query = preg_replace('/ AND $/', '', $query);
                    if (!preg_match('/WHERE $/', $query)) {
                        $innerResult = mysql_query($query, $this->dbConnection);
                        if (!$innerResult) {
                            $errors[] = "Error executing upgrade code: $query";
                        }
                    }
                }
            }
        }
    }
    
    /**
     * A private method for running a series of queries. Checks for various
     * error conditions, and may ignore errors, depending on the query.
     *
     * @private
     * @param array $queries A reference to an array of SQL queries to run.
     * @param array $errors A reference to an array that should contain any errors 
     *                      resulting from running the queries, excluding certain
     *                      cases.
     */
    function _runQueries(&$queries, &$errors)
    {
        foreach ($queries as $query) {
            $result = mysql_query($query, $this->dbConnection);
            if (!$result) {
                $error = mysql_errno($this->dbConnection);
                if (preg_match('/^ALTER\s+TABLE\s+\w+\s+ADD\s+(?!INDEX)(?!KEY)(?!PRIMARY\s+KEY)/', $query)) {
                    // May be error 1060, caused by trying to re-add a column
                    if ($error == 1060) {
                        continue;
                    }
                } else if (preg_match('/^ALTER\s+TABLE\s+\w+\s+ADD\s+(INDEX|KEY)/', $query)) {
                    // May be error 1061, caused by trying to re-add an index or key
                    if ($error == 1061) {
                        continue;
                    }
                } else if (preg_match('/^ALTER\s+TABLE\s+\w+\s+ADD\s+PRIMARY\s+KEY/', $query)) {
                    // May be error 1068, caused by trying to re-add a primary key
                    if ($error == 1068) {
                        continue;
                    }
                } else if (preg_match('/^ALTER\s+TABLE\s+\w+\s+CHANGE/', $query)) {
                    // May be error 1054, caused by trying to change a non-existant column
                    if ($error == 1054) {
                        continue;
                    }
                } else if (preg_match('/^ALTER\s+TABLE\s+\w+\s+DROP(?!PRIMARY\s+KEY)/', $query)) {
                    // May be error 1091, caused by trying to drop a non-existant index, key, or column
                    if ($error == 1091) {
                        continue;
                    }
                } else if (preg_match('/^DELETE\s+FROM/', $query)) {
                    // May be error 1054, caused by tring to delete where a column has already been dropped
                    if ($error == 1054) {
                        continue;
                    }
                }
                $errors[] = "Error executing upgrade code: $query";
            }
        }
    }
    
}

?>
